/**
 * Copyright (c) 2017 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Cocoa

class Document: NSDocument {

  //convenience to avoid digging through windowControllers array
  var primaryWindowController: WindowController?
  //temporary storage 
  var textData: Data?

  override class func autosavesInPlace() -> Bool {
    return true
  }

  override func makeWindowControllers() {
    // Returns the Storyboard that contains your Document window.
    let storyboard = NSStoryboard(name: MainStoryboardIdentifiers.name, bundle: nil)
    let windowController = storyboard.instantiateController(withIdentifier: MainStoryboardIdentifiers.documentWindow) as! WindowController
    primaryWindowController = windowController
    self.addWindowController(windowController)
    setWindowTextData(data: textData)
  }

  override func data(ofType typeName: String) throws -> Data {

    if let windowController = primaryWindowController, let viewController = windowController.contentViewController as? ViewController {
      return viewController.textData
    }

    throw NSError(domain: "BabyScript", code: 0, userInfo: nil)
  }

  override func read(from data: Data, ofType typeName: String) throws {
    textData = data //hang on to this data because the window might not exist yet
    setWindowTextData(data: data)
  }

  private func setWindowTextData(data: Data?) {
    if let data = data, let windowController = primaryWindowController, let viewController = windowController.contentViewController as? ViewController {
      viewController.textData = data
    }
  }

}
