/**
 * Copyright (c) 2017 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Cocoa

class ViewController: NSViewController {

  @IBOutlet var text: NSTextView!

  // The NSTextView can supply RFTD data or be injected with it.
  var textData: Data {

    get {
      let range = NSRange(location: 0, length: text.attributedString().length)
      let rtfd = text.attributedString().rtfd(from: range, documentAttributes:[NSDocumentTypeDocumentOption: NSRTFDTextDocumentType])
      return rtfd ?? Data()
    }

    set(inData) {
      let range = NSRange(location: 0, length: text.attributedString().length)
      text.replaceCharacters(in: range, withRTFD: inData)
    }

  }

  @IBAction func showWordCountWindow(_ sender: AnyObject) {

    // 1
    let storyboard = NSStoryboard(name: MainStoryboardIdentifiers.name, bundle: nil)
    let wordCountWindowController = storyboard.instantiateController(withIdentifier: MainStoryboardIdentifiers.wordCountWindow) as! NSWindowController

    if let wordCountWindow = wordCountWindowController.window, let textStorage = text.textStorage {

      // 2
      let wordCountViewController = wordCountWindow.contentViewController as! WordCountViewController
      wordCountViewController.wordCount = textStorage.words.count
      wordCountViewController.paragraphCount = textStorage.paragraphs.count

      // 3
      let application = NSApplication.shared()
      application.runModal(for: wordCountWindow)
    }
  }

}
